/*
 * Copyright (C) 2010 Teleal GmbH, Switzerland
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Affero General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Affero General Public License for more details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package org.teleal.cling.test.data;

import org.teleal.cling.model.Resource;
import org.teleal.cling.model.meta.Device;
import org.teleal.cling.model.meta.DeviceIdentity;
import org.teleal.cling.model.meta.DeviceService;
import org.teleal.cling.model.meta.Icon;
import org.teleal.cling.model.meta.LocalDevice;
import org.teleal.cling.model.meta.LocalDeviceService;
import org.teleal.cling.model.meta.LocalService;

import java.net.URI;

import static org.testng.Assert.assertEquals;

/**
 * @author Christian Bauer
 */
public class SampleDeviceRootLocal extends SampleDeviceRoot {

    public SampleDeviceRootLocal(DeviceIdentity identity, LocalService service, Device[] embeddedDevices) {
        super(identity, service, embeddedDevices);
    }

    @Override
    public LocalService getService() {
        return (LocalService) super.getService();
    }

    @Override
    public DeviceService[] getDeviceServices() {
        return new DeviceService[]{
                new LocalDeviceService(
                        getServiceOneType(),
                        getServiceOneId(),
                        getService()
                )
        };
    }

    public static void assertLocalResourcesMatch(LocalDevice d) {
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/desc.xml")).getType(),
                Resource.Type.DEVICE_DESCRIPTOR
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/icon.png")).getType(),
                Resource.Type.ICON
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/icon2.png")).getType(),
                Resource.Type.ICON
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/service/upnp-org/MY-SERVICE-123/desc.xml")).getType(),
                Resource.Type.SERVICE_DESCRIPTOR
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/service/upnp-org/MY-SERVICE-123/control")).getType(),
                Resource.Type.CONTROL
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/service/upnp-org/MY-SERVICE-123/events")).getType(),
                Resource.Type.EVENT_SUBSCRIPTION
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/icon3.png")).getType(),
                Resource.Type.ICON
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/service/upnp-org/MY-SERVICE-456/desc.xml")).getType(),
                Resource.Type.SERVICE_DESCRIPTOR
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/service/upnp-org/MY-SERVICE-456/control")).getType(),
                Resource.Type.CONTROL
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/service/upnp-org/MY-SERVICE-456/events")).getType(),
                Resource.Type.EVENT_SUBSCRIPTION
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/embedded/MY-DEVICE-789/service/upnp-org/MY-SERVICE-789/desc.xml")).getType(),
                Resource.Type.SERVICE_DESCRIPTOR
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/embedded/MY-DEVICE-789/service/upnp-org/MY-SERVICE-789/control")).getType(),
                Resource.Type.CONTROL
        );
        assertEquals(
                d.getLocalResource(URI.create("/upnp/device/MY-DEVICE-123/embedded/MY-DEVICE-456/embedded/MY-DEVICE-789/service/upnp-org/MY-SERVICE-789/events")).getType(),
                Resource.Type.EVENT_SUBSCRIPTION
        );

    }

}
